@@ -100,14 +100,14 @@ struct dhcp_op {
 };
 
 static const struct dhcp_op dhcp_ops[] = {
-	{ DHCP_DISCOVER, "DHCP_DISCOVER" },
-	{ DHCP_OFFER,    "DHCP_OFFER" },
-	{ DHCP_REQUEST,  "DHCP_REQUEST" },
-	{ DHCP_DECLINE,  "DHCP_DECLINE" },
-	{ DHCP_ACK,      "DHCP_ACK" },
-	{ DHCP_NAK,      "DHCP_NAK" },
-	{ DHCP_RELEASE,  "DHCP_RELEASE" },
-	{ DHCP_INFORM,   "DHCP_INFORM" },
+	{ DHCP_DISCOVER, "DISCOVER" },
+	{ DHCP_OFFER,    "OFFER" },
+	{ DHCP_REQUEST,  "REQUEST" },
+	{ DHCP_DECLINE,  "DECLINE" },
+	{ DHCP_ACK,      "ACK" },
+	{ DHCP_NAK,      "NAK" },
+	{ DHCP_RELEASE,  "RELEASE" },
+	{ DHCP_INFORM,   "INFORM" },
 	{ 0, NULL }
 };
 
@@ -197,8 +197,13 @@ handle_exit_timeout(_unused void *arg)
 	int timeout;
 
 	syslog(LOG_ERR, "timed out");
-	if (!(options & DHCPCD_TIMEOUT_IPV4LL))
-		exit(EXIT_FAILURE);
+	if (!(options & DHCPCD_TIMEOUT_IPV4LL)) {
+		if (options & DHCPCD_MASTER) {
+			daemonise();
+			return;
+		} else
+			exit(EXIT_FAILURE);
+	}
 	options &= ~DHCPCD_TIMEOUT_IPV4LL;
 	timeout = (PROBE_NUM * PROBE_MAX) + PROBE_WAIT + 1;
 	syslog(LOG_WARNING, "allowing %d seconds for IPv4LL timeout", timeout);
@@ -218,26 +223,6 @@ drop_config(struct interface *iface, const char *reason)
 	iface->state->lease.addr.s_addr = 0;
 }
 
-void
-close_sockets(struct interface *iface)
-{
-	if (iface->arp_fd != -1) {
-		delete_event(iface->arp_fd);
-		close(iface->arp_fd);
-		iface->arp_fd = -1;
-	}
-	if (iface->raw_fd != -1) {
-		delete_event(iface->raw_fd);
-		close(iface->raw_fd);
-		iface->raw_fd = -1;
-	}
-	if (iface->udp_fd != -1) {
-		/* we don't listen to events on the udp */
-		close(iface->udp_fd);
-		iface->udp_fd = -1;
-	}
-}
-
 struct interface *
 find_interface(const char *ifname)
 {
@@ -320,6 +305,10 @@ send_message(struct interface *iface, int type,
 		    iface->name, get_dhcp_op(type), state->xid,
 		    timeval_to_double(&tv));
 	}
+
+	/* Ensure sockets are open. */
+	open_sockets(iface);
+
 	/* If we couldn't open a UDP port for our IP address
 	 * then we cannot renew.
 	 * This could happen if our IP was pulled out from underneath us.
@@ -340,8 +329,10 @@ send_message(struct interface *iface, int type,
 		to.s_addr = 0;
 	if (to.s_addr && to.s_addr != INADDR_BROADCAST) {
 		r = send_packet(iface, to, (uint8_t *)dhcp, len);
-		if (r == -1)
+		if (r == -1) {
 			syslog(LOG_ERR, "%s: send_packet: %m", iface->name);
+			close_sockets(iface);
+		}
 	} else {
 		len = make_udp_packet(&udp, (uint8_t *)dhcp, len, from, to);
 		r = send_raw_packet(iface, ETHERTYPE_IP, udp, len);
@@ -361,6 +352,7 @@ send_message(struct interface *iface, int type,
 		}
 	}
 	free(dhcp);
+
 	/* Even if we fail to send a packet we should continue as we are
 	 * as our failure timeouts will change out codepath when needed. */
 	if (callback)
@@ -420,8 +412,10 @@ start_expire(void *arg)
 
 static void
 log_dhcp(int lvl, const char *msg,
-    const struct interface *iface, const struct dhcp_message *dhcp)
+    const struct interface *iface, const struct dhcp_message *dhcp,
+    const struct in_addr *from)
 {
+	const char *tfrom;
 	char *a;
 	struct in_addr addr;
 	int r;
@@ -433,21 +427,24 @@ log_dhcp(int lvl, const char *msg,
 		a = xstrdup(inet_ntoa(addr));
 	} else
 		a = NULL;
+
+	tfrom = "from";
 	r = get_option_addr(&addr, dhcp, DHO_SERVERID);
 	if (dhcp->servername[0] && r == 0)
-		syslog(lvl, "%s: %s %s from %s `%s'", iface->name, msg, a,
-		    inet_ntoa(addr), dhcp->servername);
-	else if (r == 0) {
+		syslog(lvl, "%s: %s %s %s %s `%s'", iface->name, msg, a,
+		    tfrom, inet_ntoa(addr), dhcp->servername);
+	else {
+		if (r != 0) {
+			tfrom = "via";
+			addr = *from;
+		}
 		if (a == NULL)
-			syslog(lvl, "%s: %s from %s",
-			    iface->name, msg, inet_ntoa(addr));
+			syslog(lvl, "%s: %s %s %s",
+			    iface->name, msg, tfrom, inet_ntoa(addr));
 		else
-			syslog(lvl, "%s: %s %s from %s",
-			    iface->name, msg, a, inet_ntoa(addr));
-	} else if (a != NULL)
-		syslog(lvl, "%s: %s %s", iface->name, msg, a);
-	else
-		syslog(lvl, "%s: %s", iface->name, msg);
+			syslog(lvl, "%s: %s %s %s %s",
+			    iface->name, msg, a, tfrom, inet_ntoa(addr));
+	}
 	free(a);
 }
 
@@ -476,7 +473,7 @@ whitelisted_ip(const struct if_options *ifo, in_addr_t addr)
 }
 
 static void
-handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
+handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp, const struct in_addr *from)
 {
 	struct if_state *state = iface->state;
 	struct if_options *ifo = state->options;
@@ -498,20 +495,16 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 		if (has_option_mask(ifo->requiremask, DHO_SERVERID) &&
 		    get_option_addr(&addr, dhcp, DHO_SERVERID) == -1)
 		{
-			log_dhcp(LOG_WARNING, "reject NAK", iface, dhcp);
+			log_dhcp(LOG_WARNING, "reject NAK", iface, dhcp, from);
 			return;
 		}
 		/* We should restart on a NAK */
-		log_dhcp(LOG_WARNING, "NAK:", iface, dhcp);
+		log_dhcp(LOG_WARNING, "NAK:", iface, dhcp, from);
 		if (!(options & DHCPCD_TEST)) {
 			drop_config(iface, "NAK");
 			unlink(iface->leasefile);
 		}
-		delete_event(iface->raw_fd);
-		close(iface->raw_fd);
-		iface->raw_fd = -1;
-		close(iface->udp_fd);
-		iface->udp_fd = -1;
+		close_sockets(iface);
 		/* If we constantly get NAKS then we should slowly back off */
 		add_timeout_sec(state->nakoff, start_interface, iface);
 		state->nakoff *= 2;
@@ -529,7 +522,7 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 			 * To ignore bootp, require dhcp_message_type instead. */
 			if (type == 0 && i == DHO_SERVERID)
 				continue;
-			log_dhcp(LOG_WARNING, "reject DHCP", iface, dhcp);
+			log_dhcp(LOG_WARNING, "reject DHCP", iface, dhcp, from);
 			return;
 		}
 	}		
@@ -546,7 +539,7 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 		if (type == 0 ||
 		    get_option_addr(&lease->server, dhcp, DHO_SERVERID) != 0)
 			lease->server.s_addr = INADDR_ANY;
-		log_dhcp(LOG_INFO, "offered", iface, dhcp);
+		log_dhcp(LOG_INFO, "offered", iface, dhcp, from);
 		free(state->offer);
 		state->offer = dhcp;
 		*dhcpp = NULL;
@@ -575,18 +568,20 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 
 	if (type) {
 		if (type == DHCP_OFFER) {
-			log_dhcp(LOG_INFO, "ignoring offer of", iface, dhcp);
+			log_dhcp(LOG_INFO, "ignoring offer of",
+			    iface, dhcp, from);
 			return;
 		}
 
 		/* We should only be dealing with acks */
 		if (type != DHCP_ACK) {
-			log_dhcp(LOG_ERR, "not ACK or OFFER", iface, dhcp);
+			log_dhcp(LOG_ERR, "not ACK or OFFER",
+			    iface, dhcp, from);
 			return;
 		}
 
 		if (!(ifo->options & DHCPCD_INFORM))
-			log_dhcp(LOG_INFO, "acknowledged", iface, dhcp);
+			log_dhcp(LOG_INFO, "acknowledged", iface, dhcp, from);
 	}
 
 	/* BOOTP could have already assigned this above, so check we still
@@ -702,7 +697,7 @@ handle_dhcp_packet(void *arg)
 			    hwaddr_ntoa(dhcp->chaddr, sizeof(dhcp->chaddr)));
 			continue;
 		}
-		handle_dhcp(iface, &dhcp);
+		handle_dhcp(iface, &dhcp, &from);
 		if (iface->raw_fd == -1)
 			break;
 	}
@@ -710,25 +705,6 @@ handle_dhcp_packet(void *arg)
 	free(dhcp);
 }
 
-static void
-open_sockets(struct interface *iface)
-{
-	if (iface->raw_fd == -1) {
-		if (open_socket(iface, ETHERTYPE_IP) == -1)
-			syslog(LOG_ERR, "%s: open_socket: %m", iface->name);
-		else
-			add_event(iface->raw_fd, handle_dhcp_packet, iface);
-	}
-	if (iface->udp_fd == -1 &&
-	    iface->addr.s_addr != 0 &&
-	    iface->state->new != NULL &&
-	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
-	{
-		if (open_udp_socket(iface) == -1 && errno != EADDRINUSE)
-			syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
-	}
-}
-
 static void
 send_release(struct interface *iface)
 {
@@ -739,7 +715,6 @@ send_release(struct interface *iface)
 	{
 		syslog(LOG_INFO, "%s: releasing lease of %s",
 		    iface->name, inet_ntoa(iface->state->lease.addr));
-		open_sockets(iface);
 		iface->state->xid = dhcp_xid(iface);
 		send_message(iface, DHCP_RELEASE, NULL);
 		/* Give the packet a chance to go before dropping the ip */
@@ -754,7 +729,6 @@ send_release(struct interface *iface)
 void
 send_decline(struct interface *iface)
 {
-	open_sockets(iface);
 	send_message(iface, DHCP_DECLINE, NULL);
 }
 
@@ -796,6 +770,7 @@ configure_interface1(struct interface *iface)
 	}
 
 	free(iface->clientid);
+	iface->clientid = NULL;
 	if (*ifo->clientid) {
 		iface->clientid = xmalloc(ifo->clientid[0] + 1);
 		memcpy(iface->clientid, ifo->clientid, ifo->clientid[0] + 1);
@@ -922,7 +897,6 @@ start_discover(void *arg)
 
 	iface->state->state = DHS_DISCOVER;
 	iface->state->xid = dhcp_xid(iface);
-	open_sockets(iface);
 	delete_timeout(NULL, iface);
 	if (ifo->fallback)
 		add_timeout_sec(ifo->timeout, start_fallback, iface);
@@ -956,7 +930,6 @@ start_renew(void *arg)
 	    iface->name, inet_ntoa(iface->state->lease.addr));
 	iface->state->state = DHS_RENEW;
 	iface->state->xid = dhcp_xid(iface);
-	open_sockets(iface);
 	send_renew(iface);
 }
 
@@ -970,8 +943,6 @@ start_rebind(void *arg)
 	iface->state->state = DHS_REBIND;
 	delete_timeout(send_renew, iface);
 	iface->state->lease.server.s_addr = 0;
-	if (iface->raw_fd == -1)
-		open_sockets(iface);
 	send_rebind(iface);
 }
 
@@ -1056,7 +1027,6 @@ start_inform(struct interface *iface)
 
 	iface->state->state = DHS_INFORM;
 	iface->state->xid = dhcp_xid(iface);
-	open_sockets(iface);
 	send_inform(iface);
 }
 
@@ -1103,7 +1073,6 @@ start_reboot(struct interface *iface)
 	else if (!(ifo->options & DHCPCD_INFORM &&
 		options & (DHCPCD_MASTER | DHCPCD_DAEMONISED)))
 		add_timeout_sec(ifo->reboot, start_expire, iface);
-	open_sockets(iface);
 	/* Don't bother ARP checking as the server could NAK us first. */
 	if (ifo->options & DHCPCD_INFORM)
 		send_inform(iface);
@@ -1185,7 +1154,8 @@ start_interface(void *arg)
 			    now.tv_sec - st.st_mtime)
 			{
 				syslog(LOG_DEBUG,
-				    "%s: discarding expired lease", iface->name);
+				    "%s: discarding expired lease",
+				    iface->name);
 				free(iface->state->offer);
 				iface->state->offer = NULL;
 				iface->state->lease.addr.s_addr = 0;
@@ -1345,7 +1315,6 @@ handle_ifa(int type, const char *ifname,
 			    dst ? dst->s_addr : INADDR_ANY;
 			ifp->addr = *addr;
 			ifp->net = *net;
-			open_sockets(ifp);
 			send_inform(ifp);
 		}
 		break;
@@ -1360,71 +1329,8 @@ handle_link(_unused void *arg)
 		syslog(LOG_ERR, "manage_link: %m");
 }
 
-/* ARGSUSED */
 static void
-handle_signal(_unused void *arg)
-{
-	struct interface *iface, *ifl;
-	int sig = signal_read();
-	int do_release = 0;
-
-	switch (sig) {
-	case SIGINT:
-		syslog(LOG_INFO, "received SIGINT, stopping");
-		break;
-	case SIGTERM:
-		syslog(LOG_INFO, "received SIGTERM, stopping");
-		break;
-	case SIGALRM:
-		syslog(LOG_INFO, "received SIGALRM, rebinding lease");
-		for (iface = ifaces; iface; iface = iface->next)
-			start_interface(iface);
-		return;
-	case SIGHUP:
-		syslog(LOG_INFO, "received SIGHUP, releasing lease");
-		do_release = 1;
-		break;
-	case SIGUSR1:
-		syslog(LOG_INFO, "received SIGUSR, reconfiguring");
-		for (iface = ifaces; iface; iface = iface->next)
-			if (iface->state->new)
-				configure(iface);
-		return;
-	case SIGPIPE:
-		syslog(LOG_WARNING, "received SIGPIPE");
-		return;
-	default:
-		syslog(LOG_ERR,
-		    "received signal %d, but don't know what to do with it",
-		    sig);
-		return;
-	}
-
-	if (options & DHCPCD_TEST)
-		exit(EXIT_FAILURE);
-
-	/* As drop_config could re-arrange the order, we do it like this. */
-	for (;;) {
-		/* Be sane and drop the last config first */
-		ifl = NULL;
-		for (iface = ifaces; iface; iface = iface->next) {
-			if (iface->next == NULL)
-				break;
-			ifl = iface;
-		}
-		if (iface == NULL)
-			break;
-		if (iface->carrier != LINK_DOWN &&
-		    (do_release ||
-			iface->state->options->options & DHCPCD_RELEASE))
-			send_release(iface);
-		stop_interface(iface);
-	}
-	exit(EXIT_FAILURE);
-}
-
-static void
-reconf_reboot(struct interface *iface, int argc, char **argv)
+if_reboot(struct interface *iface, int argc, char **argv)
 {
 	const struct if_options *ifo;
 	int opt;
@@ -1447,10 +1353,148 @@ reconf_reboot(struct interface *iface, int argc, char **argv)
 	start_interface(iface);
 }
 
+static void
+reconf_reboot(int action, int argc, char **argv, int oi)
+{
+	struct interface *ifl, *ifn, *ifp, *ifs, *ift;
+	
+	ifs = discover_interfaces(argc - oi, argv + oi);
+	if (ifs == NULL)
+		return;
+
+	/* Remove any old interfaces */
+	if (ifaces) {
+		for (ifl = NULL; ifl != ifaces;) {
+			/* Work our way backwards */
+			for (ifp = ifaces; ifp; ifp = ifp->next)
+				if (ifp->next == ifl) {
+					ifl = ifp;
+					break;
+				}
+			for (ifn = ifs; ifn; ifn = ifn->next)
+				if (strcmp(ifn->name, ifp->name) == 0)
+					break;
+			if (ifn == NULL) {
+				ifl = ifp->next;
+				stop_interface(ifp);
+			}
+		}
+	}
+	
+	for (ifp = ifs; ifp && (ift = ifp->next, 1); ifp = ift) {
+		ifl = NULL;
+		for (ifn = ifaces; ifn; ifn = ifn->next) {
+			if (strcmp(ifn->name, ifp->name) == 0)
+				break;
+			ifl = ifn;
+		}
+		if (ifn) {
+			if (action)
+				if_reboot(ifn, argc, argv);
+			else if (ifn->state->new)
+				configure(ifn);
+			free_interface(ifp);
+		} else {
+			ifp->next = NULL;
+			init_state(ifp, argc, argv);
+			start_interface(ifp);
+			if (ifl)
+				ifl->next = ifp;
+			else
+				ifaces = ifp;
+		}
+	}
+
+	sort_interfaces();
+}
+
+/* ARGSUSED */
+static void
+handle_signal(_unused void *arg)
+{
+	struct interface *ifp, *ifl;
+	struct if_options *ifo;
+	int sig = signal_read();
+	int do_release, do_rebind, i;
+
+	do_rebind = do_release = 0;
+	switch (sig) {
+	case SIGINT:
+		syslog(LOG_INFO, "received SIGINT, stopping");
+		break;
+	case SIGTERM:
+		syslog(LOG_INFO, "received SIGTERM, stopping");
+		break;
+	case SIGALRM:
+		syslog(LOG_INFO, "received SIGALRM, rebinding");
+		for (i = 0; i < ifac; i++)
+			free(ifav[i]);
+		free(ifav);
+		ifav = NULL;
+		ifac = 0;
+		for (i = 0; i < ifdc; i++)
+			free(ifdv[i]);
+		free(ifdv);
+		ifdc = 0;
+		ifdv = NULL;
+		ifo = read_config(cffile, NULL, NULL, NULL);
+		add_options(ifo, margc, margv);
+		/* We need to preserve these two options. */
+		if (options & DHCPCD_MASTER)
+			ifo->options |= DHCPCD_MASTER;
+		if (options & DHCPCD_DAEMONISED)
+			ifo->options |= DHCPCD_DAEMONISED;
+		options = ifo->options;
+		free_options(ifo);
+		reconf_reboot(1, 0, NULL, 0);
+		return;
+	case SIGHUP:
+		syslog(LOG_INFO, "received SIGHUP, releasing");
+		do_release = 1;
+		break;
+	case SIGUSR1:
+		syslog(LOG_INFO, "received SIGUSR, reconfiguring");
+		for (ifp = ifaces; ifp; ifp = ifp->next)
+			if (ifp->state->new)
+				configure(ifp);
+		return;
+	case SIGPIPE:
+		syslog(LOG_WARNING, "received SIGPIPE");
+		return;
+	default:
+		syslog(LOG_ERR,
+		    "received signal %d, but don't know what to do with it",
+		    sig);
+		return;
+	}
+
+	if (options & DHCPCD_TEST)
+		exit(EXIT_FAILURE);
+
+	/* As drop_config could re-arrange the order, we do it like this. */
+	for (;;) {
+		/* Be sane and drop the last config first */
+		ifl = NULL;
+		for (ifp = ifaces; ifp; ifp = ifp->next) {
+			if (ifp->next == NULL)
+				break;
+			ifl = ifp;
+		}
+		if (ifp == NULL)
+			break;
+		if (ifp->carrier != LINK_DOWN &&
+		    (do_release ||
+			ifp->state->options->options & DHCPCD_RELEASE))
+			send_release(ifp);
+		stop_interface(ifp);
+	}
+	exit(EXIT_FAILURE);
+}
+
 int
 handle_args(struct fd_list *fd, int argc, char **argv)
 {
-	struct interface *ifs, *ifp, *ifl, *ifn, *ift;
+	struct interface *ifp;
 	int do_exit = 0, do_release = 0, do_reboot = 0, do_reconf = 0;
 	int opt, oi = 0;
 	ssize_t len;
@@ -1573,35 +1617,49 @@ handle_args(struct fd_list *fd, int argc, char **argv)
 		return 0;
 	}
 
-	if ((ifs = discover_interfaces(argc - optind, argv + optind))) {
-		for (ifp = ifs; ifp && (ift = ifp->next, 1); ifp = ift) {
-			ifl = NULL;
-			for (ifn = ifaces; ifn; ifn = ifn->next) {
-				if (strcmp(ifn->name, ifp->name) == 0)
-					break;
-				ifl = ifn;
-			}
-			if (ifn) {
-				if (do_reboot)
-					reconf_reboot(ifn, argc, argv);
-				else if (do_reconf && ifn->state->new)
-					configure(ifn);
-				free_interface(ifp);
-			} else {
-				ifp->next = NULL;
-				init_state(ifp, argc, argv);
-				start_interface(ifp);
-				if (ifl)
-					ifl->next = ifp;
-				else
-					ifaces = ifp;
-			}
-		}
-		sort_interfaces();
-	}
+	reconf_reboot(do_reboot, argc, argv, optind);
 	return 0;
 }
 
+void
+open_sockets(struct interface *iface)
+{
+	if (iface->raw_fd == -1) {
+		if (open_socket(iface, ETHERTYPE_IP) == -1)
+			syslog(LOG_ERR, "%s: open_socket: %m", iface->name);
+		else
+			add_event(iface->raw_fd, handle_dhcp_packet, iface);
+	}
+	if (iface->udp_fd == -1 &&
+	    iface->addr.s_addr != 0 &&
+	    iface->state->new != NULL &&
+	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
+	{
+		if (open_udp_socket(iface) == -1 && errno != EADDRINUSE)
+			syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
+	}
+}
+
+void
+close_sockets(struct interface *iface)
+{
+	if (iface->arp_fd != -1) {
+		delete_event(iface->arp_fd);
+		close(iface->arp_fd);
+		iface->arp_fd = -1;
+	}
+	if (iface->raw_fd != -1) {
+		delete_event(iface->raw_fd);
+		close(iface->raw_fd);
+		iface->raw_fd = -1;
+	}
+	if (iface->udp_fd != -1) {
+		/* we don't listen to events on the udp */
+		close(iface->udp_fd);
+		iface->udp_fd = -1;
+	}
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1613,7 +1671,7 @@ main(int argc, char **argv)
 	struct timespec ts;
 
 	closefrom(3);
-	openlog(PACKAGE, LOG_PERROR, LOG_DAEMON);
+	openlog(PACKAGE, LOG_PERROR | LOG_PID, LOG_DAEMON);
 	setlogmask(LOG_UPTO(LOG_INFO));
 
 	/* Test for --help and --version */
@@ -1679,7 +1737,7 @@ main(int argc, char **argv)
 
 	if (options & DHCPCD_DEBUG)
 		setlogmask(LOG_UPTO(LOG_DEBUG));
-	else if (options & DHCPCD_QUIET)
+	if (options & DHCPCD_QUIET)
 		close(STDERR_FILENO);
 
 	if (!(options & DHCPCD_TEST)) {
@@ -1881,12 +1939,12 @@ main(int argc, char **argv)
 		{
 			syslog(LOG_WARNING, "no interfaces have a carrier");
 			daemonise();
-		} else if (options & DHCPCD_DAEMONISE && ifo->timeout > 0) {
+		} else if (ifo->timeout > 0) {
 			if (options & DHCPCD_IPV4LL)
 				options |= DHCPCD_TIMEOUT_IPV4LL;
-			add_timeout_sec(ifo->timeout, handle_exit_timeout, NULL);
+			add_timeout_sec(ifo->timeout, handle_exit_timeout,
+			    NULL);
 		}
-
 	}
 	free_options(ifo);
 
